<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Base\PropertyMapConfig;
use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\Support\Storage\Cache;

class OcPropertyMapConfig implements PropertyMapConfig
{
    private static $config;
    private static $configFilePath = ABSPATH . 'config.json';

    private function getConfigFromCache(): array
    {
        $key = 'ew_config_' . GIT_COMMIT;

        if (($config = Cache::get($key)) === false) {
            $config = $this->getConfigFromFile();

            Cache::set($key, $config, WEEK_IN_SECONDS);
        }

        return $config;
    }

    private function getConfigFromFile(): array
    {
        return json_decode(file_get_contents(static::$configFilePath), true);
    }

    private function getConfig(): array
    {
        if (static::$config === null) {
            if (env('APP_ENV') === 'production') {
                static::$config = $this->getConfigFromCache();
            } else {
                static::$config = $this->getConfigFromFile();
            }
        }

        return static::$config;
    }

    public function getPropertyMap(): array
    {
        return $this->getConfigProperty('typePropertyMap');
    }

    public function getTypeProperty(string $type): array
    {
        $propertyMap = $this->getPropertyMap();

        return array_key_exists($type, $propertyMap) ? $propertyMap[$type] : [];
    }

    public function getPropertyClassReference(): array
    {
        return $this->getConfigProperty('typePropertyClassReference');
    }

    public function getConfigProperty(string $property): array
    {
        return Arr::get($this->getConfig(), $property, []);
    }
}
