<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Support\Arr;

/**
 * PropertyMapper
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base
 * @since   Infomaker\Everyware\Base\PropertyMapper 1.0.0
 */
class PropertyMapper
{

    /**
     * @var array
     */
    private $property_map;

    public function __construct(PropertyMapConfig $config)
    {
        $this->property_map = $config->getPropertyMap();
    }

    public function requiredProperties(string $object_map): array
    {
        if (array_key_exists($object_map, $this->property_map)) {
            return Arr::flatten($this->extractProperties($this->property_map[$object_map]));
        }

        return [];
    }

    private function extractProperties(array $properties): array
    {
        return array_map(function ($property = []) {
            $property_name = $property['name'];
            if ($this->hasReference($property)) {
                $referenced_properties = $this->requiredProperties($this->getReference($property));

                array_walk($referenced_properties, function (&$property, $key, $relation) {
                    $property = "{$relation}.$property";
                }, $property_name);

                return $referenced_properties;
            }

            return $property_name;
        }, $properties);
    }

    private function hasReference($property): bool
    {
        return isset($property['propertyMapReference']);
    }

    private function getReference($property)
    {
        return $property['propertyMapReference'];
    }
}
